/*
 Stephan Kottler
 kottlers@informatik.uni-tuebingen.de

 */

#include "Statistics.hpp"
#include "BasicSolver.hpp"
#include <signal.h>

#include "Types.hpp"
#include "IFStream.hpp"
#include "Graph.hpp"
#include "BasicEncoding.hpp"
#include "GridEdgeEncoder.hpp"
//#include "DirectGridEncoder.hpp"


/**
 * Some checks
 */

bool check_defines(){

    return
    //sizeof(void*)           ==      32/8                &&
    sizeof(int32_t)         ==      32/8                &&
    sizeof(int64_t)         ==      64/8                &&
    sizeof(uint32_t)        ==      32/8                &&
    sizeof(uint64_t)        ==      64/8                &&
    sizeof(double)          ==      64/8                &&
    sizeof(float)           ==      32/8                &&
    sizeof(Literal)         ==      sizeof(uint32_t)    &&
    sizeof(Variable)        ==      sizeof(uint32_t)    &&
    sizeof(float)           ==      sizeof(uint32_t)    &&
    ((int)false)            ==      0                   &&
    ((int)true)             ==      1;
}


OFStream 	console_out(stderr);
Settings    settings;
Statistics  statistics(settings);
BasicSolver solver(settings,statistics,&console_out);


static void SIGQUIT_handler(int signum){
    console_out.writeNewLine();
    console_out.writeString("|-------------------------------------------------------------------------------------------------");
    console_out.writeCenteredToBG("!!! INFO !!!");
    statistics.printFinalStatistics(console_out,&solver);
    console_out.writeCenteredToBG("!!! INFO !!!");
    console_out.writeString("|-------------------------------------------------------------------------------------------------");

};

static void SIGINT_handler(int signum) {
    console_out.writeNewLine();
    console_out.writeString("|-------------------------------------------------------------------------------------------------");
    console_out.writeCenteredToBG("!!! KILLED !!!");
    statistics.printFinalStatistics(console_out,&solver);
    console_out.writeCenteredToBG("!!! KILLED !!!");
    console_out.writeString("|-------------------------------------------------------------------------------------------------");
    exit(1);
};


int main(int argc, char** argv){
	assert((print_out("Asserts will be evaluated\n"),true));
	costly_assert((print_out("Costly Asserts will be evaluated\n"),true));

    assert(check_defines());

    if(!settings.read(argc,argv)){
        settings.printUsage();
        return EXIT_FAILURE;
    }

    signal(SIGINT,SIGINT_handler);
    signal(SIGQUIT,SIGQUIT_handler);



	console_out.writeNewLine();
	console_out.writeString(
		"|#################################################################################################|\n");
	console_out.setBackgroundPattern(
		"|                                                                                                 |\n");


    TrBool result = UNKNOWN_TrBool;
    char* result_txt = (char*)"UNKNOWN";
    char* instance;

    Translator encodeTranslate;

    assert(settings.tgf_file_name != NULL);

	instance = basename(settings.tgf_file_name);
	IFStream in(settings.tgf_file_name);
	if(in.isBad()){
		print_out("Could not open TGF file %s ", settings.tgf_file_name);
		return EXIT_FAILURE;
	}

	Graph graph;
	if(!graph.parseTGF(in)) return EXIT_FAILURE;
	statistics.vertex_count = graph.vertexCount();
	statistics.edge_count	= graph.edgeCount();

	console_out.writeString("| Finished parsing of TGF with %d vertices and %d edges ",
								statistics.vertex_count,statistics.edge_count);


	/*switch(settings.encoding_type){
	case Edge:{*/
	GridEdgeEncode grid(graph,solver,encodeTranslate);
	if(grid.initGrid(settings.grid_width,settings.grid_height) && grid.encode()){
		settings.grid_height = grid.rowCount(); settings.grid_width = grid.colCount();
	}else{result = FALSE_TrBool;}


	console_out.writeString("| Finished encoding with %d rows and %d columns",settings.grid_height,settings.grid_width);


	if(settings.encoding_info_out != NULL){
		OFStream encode_out(settings.encoding_info_out);
		for(uint i = 0; i != encodeTranslate.varsCount();++i){
			encode_out.writeString("%d: %s \n",i+1,encodeTranslate.get(i).c_str());
		}
	}


    statistics.parse_timestmp = statistics.cpuUserTime() ;

  	if(settings.dimacs_out != NULL){
  		OFStream out(settings.dimacs_out);
  		if(out.isBad()){
  			print_out("Could not open file for output %s ", settings.dimacs_out);
  		   	return EXIT_FAILURE;
  		}
  		solver.printDIMACS(&out, NULL);

  	}else{
  		result = result.isUnknown() ? solver.isSAT( (double)settings.timeout): result;
  	}

    if(result.isTrue()){
    	result_txt = (char*)"SAT";
    	//print_out("SATISFIABLE\n");
    	solver.printModel(encodeTranslate);

    }else if(result.isFalse()){
    	result_txt =(char*)"UNSAT";
    	//print_out("UNSATISFIABLE\n");
    }


	console_out.writeString(
		"|Instance %s is %s", basename(instance ),result_txt );
	statistics.printFinalStatistics(console_out,&solver);
	console_out.writeString(
				"|#################################################################################################|\n");


    /*if(settings.csv_file_name != NULL){
    	OFStream out(settings.csv_file_name,false);
        statistics.printCSV(out,&solver,result_txt);
    }*/

/*
#ifdef NDEBUG
    exit(0);
#endif
*/

    return 0;
}




const char*  sapperlot_cpp_version(){return "SApperloT.cpp:      $Revision: 1.11 $";}


