/*
 Stephan Kottler
 kottlers@informatik.uni-tuebingen.de
 *
 */

#ifndef SETTINGS_HPP_
#define SETTINGS_HPP_
#include "Random.hpp"
#include "OFStream.hpp"
#include <stdio.h>
#include <cassert>
#include <stdint.h>
#include <vector>
#include <string>
#include <utility>
#include <string>
#include "LazyFraction.hpp"

typedef LazyFraction<7,uint16_t,uint16_t> cls_act_t;
typedef LazyFraction<7,uint16_t,uint16_t> var_act_t;
typedef LazyFraction<7,uint16_t,uint16_t> cat_act_t;


#define print_out(format, args...) ( fflush(stdout), fprintf(stderr, format, ## args), fflush(stderr) )

//#define print_file(file, format, args...) ( fprintf(file, format, ## args) )



template<class T>
struct Option{
	typedef std::pair<const char*,T>		val_type;
private:
	T* 										variable;
	const char*								command_arg;
	int										command_len;
	const char*								description;
	std::vector<val_type >					possible_vals;

public:

	Option(T* var,T default_val,const char* arg,const char* descr)
		:variable(var),command_arg(arg),command_len(strlen(arg)),description(descr){
		*variable	= default_val;};

	void addPossibleValue(val_type pval){  possible_vals.push_back(pval);}

	char* canHandle(char* arg){
		return (strncmp(arg, command_arg, command_len) == 0) ? (arg + command_len) : NULL;}

	void assign(T val){*variable = val;}

	bool findValue(char* arg){
		if(possible_vals.empty()) return false;
		for(int i = possible_vals.size();i--;){
			if(strcmp(possible_vals[i].first,arg) == 0){
				assign(possible_vals[i].second);
				return true;
			}
		}
		print_out("Unknown value %s of option %s \n",arg,command_arg);
		exit(EXIT_FAILURE);
		return false;
	}

	void printValue(std::string format){
		for(int i = possible_vals.size();i--;){
			if(*variable == possible_vals[i].second){
				print_out("\t%-*s \t%s\n",20,command_arg,possible_vals[i].first);
				return;
			}
		}
		std::string form = "\t%-*s \t" + format + "\n";
		print_out(form.c_str(),20,command_arg,*variable);
	}


	void printDescription(){
		print_out("\t%-*s \t-%s",20,command_arg,description);
		int vals = possible_vals.size();
		if(vals){
			print_out(" - possible values are: ");
			for(int i = 0; i != vals;++i) print_out(" | %s",possible_vals[i].first);
		}
		print_out("\n");
	}
};



//Polarity Mode
typedef uint PolarityMode;
const PolarityMode Polarity_False		= 0;
const PolarityMode Polarity_True		= 1;
const PolarityMode Polarity_Rand		= 2;
const PolarityMode Polarity_Prev		= 3;
const PolarityMode Polarity_Card		= 4;
const PolarityMode Polarity_Best		= 5;


//Encodings
typedef uint Encoding;
const Encoding	   Direct				= 0;
const Encoding	   Edge					= 1;
const Encoding	   Relative				= 2;


class Settings{
	//bool				print_console;
	uint				rand_seed;

public:

	Random				randy;
	bool				minimise_by_cat;
	uint				timeout;
	uint				max_literals;
	uint				restart_reset;
	PolarityMode		polarity_mode;
	//Encoding			encoding_type;

	const var_act_t   	var_decay;	// Inverse of the variable activity decay factor.
	const var_act_t   	var_min_inc;
	const var_act_t   	var_max_act;

	const cat_act_t   	cat_decay;	// Inverse of the variable activity decay factor.
	const cat_act_t   	cat_min_inc;
	const cat_act_t   	cat_max_act;

	const cls_act_t   	cls_decay;	// Inverse of the clause activity decay factor.
	const cls_act_t   	cls_min_inc;
	const cls_act_t   	cls_max_act;

	uint				random_var_freq_permil;
	uint				random_cat_freq_permil;
	uint				random_arc_check_freq_permil;

	int				conflict_max_init;
	double			conflict_max_inc;
	bool			only_bump_on_card_match;
	//bool 			bump_bin_vars;
	//bool 			bump_two_left_in_tern;
	bool 			preproc_shuffle;
	//bool 			big_simp_at_start;
	//bool			flip_first_in_preproc;
	//bool			sort_watchers;
	uint 			max_tern_closure;
	uint			preprocess_level;
	uint			preprocess_conflicts;
	uint			arc_consistenct_check;
	uint			bin_cls_to_force_closure;

	uint			grid_width;
	uint			grid_height;

	double			learntsize_factor;
	double			learntsize_inc;
	//char*			csv_file_name;
	char* 			encoding_info_out;
	char*			tgf_file_name;
	char*			dimacs_out;
	char*			solver_name;
	std::string		command_line;
	//OFStream* 		console_out;

	std::vector<Option<uint> >   int_options;
	std::vector<Option<bool> >  bool_options;
	std::vector<Option<char*> > string_options;

	Settings();
	~Settings();

	bool			read(int argc, char** argv);
	void			printUsage();
};


const int KILO = 1 << 10;
const int MEGA = 1 << 20;


inline void set_max_int(uint32_t& to_be_max, const uint32_t cand) { if(cand > to_be_max){to_be_max = cand;}}

inline uint32_t hash32(const int& x)					{ return 1 << (x & 31);}

inline uint64_t hash64(const int& x)					{ return 1 << (x & 63);}



template<class I>
inline void pushFront(std::vector<I>& vec, const I& ele){
	vec.push_back(ele);
	uint i = vec.size();
	while(--i){vec[i] = vec[i-1];}
	vec[0] = ele;
}

template<class I>
inline bool allEqual(const std::vector<I>& vec, const I& ele){
	int32_t i = vec.size();
	while(i--){if(vec[i] != ele){ return false;}}

	return true;
}

template<class E, class Pred>
inline bool find(const std::vector<E>& vec,Pred p){

	int32_t i = vec.size();
	while(i--){if(p(vec[i])){ return true;}}

	return false;
}


template<class E, class Pred>
inline void stable_remove_one(std::vector<E>& vec,Pred p){
	assert(!vec.empty());

	int32_t i = vec.size();
	while(i--){if(p(vec[i])){ ++i; break;}}
	assert( i > 0 );

	for(;i < (int32_t)vec.size();++i){vec[i-1]=vec[i];}
	vec.pop_back();
}


template<class E, class P>
inline void unstable_remove_one(std::vector<E>& vec, P p){
	assert(!vec.empty());

	for(int32_t i(vec.size());i--;){
		if(p(vec[i])){
			vec[i] = vec.back(); vec.pop_back();
			return;
	}}
	assert(false);
}

inline char* basename(char* str, char sep = '/'){

	assert(str != NULL);
	for(char* iter = str; *iter != '\0' ; ){
		if(*iter++ == sep) str = iter;
	}
	return str;
}


#ifdef  COSTLY_ASSERT
//has only an effect if NDEBUG is not defined

# define costly_assert(expr)	(assert(expr))

#else

# define costly_assert(expr)	(__ASSERT_VOID_CAST (0))

# endif

#define settings_hpp_version "Settings.hpp:     $Revision: 1.11 $"

const char*  settings_cpp_version();
#endif /*SETTINGS_HPP_*/
